@@ -68,7 +68,7 @@ def get_pybb_profile(user):
 def get_pybb_profile_model():
     from pybb import defaults
     if defaults.PYBB_PROFILE_RELATED_NAME:
-        return get_user_model()._meta.get_field_by_name(defaults.PYBB_PROFILE_RELATED_NAME)[0].model
+        return getattr(get_user_model(), defaults.PYBB_PROFILE_RELATED_NAME).related.model
     else:
         return get_user_model()
 
@@ -89,4 +89,4 @@ def get_file_path(instance, filename, to):
     """
     ext = filename.split('.')[-1]
     filename = "%s.%s" % (uuid.uuid4(), ext)
-    return os.path.join(to, filename)
\ No newline at end of file
+    return os.path.join(to, filename)
