@@ -134,7 +134,7 @@ class ForumView(RedirectToLoginMixin, PaginatorMixin, generic.ListView):
         if not perms.may_view_forum(self.request.user, self.forum):
             raise PermissionDenied
 
-        qs = self.forum.topics.order_by('-sticky', '-updated').select_related()
+        qs = self.forum.topics.order_by('-sticky', '-updated', '-id').select_related()
         qs = perms.filter_topics(self.request.user, qs)
         return qs
 
@@ -148,7 +148,7 @@ class LatestTopicsView(PaginatorMixin, generic.ListView):
     def get_queryset(self):
         qs = Topic.objects.all().select_related()
         qs = perms.filter_topics(self.request.user, qs)
-        return qs.order_by('-updated')
+        return qs.order_by('-updated', '-id')
 
 
 class TopicView(RedirectToLoginMixin, PaginatorMixin, generic.ListView):
@@ -194,7 +194,7 @@ class TopicView(RedirectToLoginMixin, PaginatorMixin, generic.ListView):
                 read_date = read_dates and max(read_dates)
                 if read_date:
                     try:
-                        first_unread_topic = self.topic.posts.filter(created__gt=read_date).order_by('created')[0]
+                        first_unread_topic = self.topic.posts.filter(created__gt=read_date).order_by('created', 'id')[0]
                     except IndexError:
                         first_unread_topic = self.topic.last_post
                 else:
@@ -249,8 +249,8 @@ class TopicView(RedirectToLoginMixin, PaginatorMixin, generic.ListView):
             ctx['first_post'] = None
         ctx['topic'] = self.topic
 
-        if self.request.user.is_authenticated() and self.topic.poll_type != Topic.POLL_TYPE_NONE and \
-           pybb_topic_poll_not_voted(self.topic, self.request.user):
+        if perms.may_vote_in_topic(self.request.user, self.topic) and \
+                pybb_topic_poll_not_voted(self.topic, self.request.user):
             ctx['poll_form'] = self.get_poll_form_class()(self.topic)
 
         return ctx
@@ -466,7 +466,7 @@ class UserPosts(PaginatorMixin, generic.ListView):
         qs = super(UserPosts, self).get_queryset()
         qs = qs.filter(user=self.user)
         qs = perms.filter_posts(self.request.user, qs).select_related('topic')
-        qs = qs.order_by('-created', '-updated')
+        qs = qs.order_by('-created', '-updated', '-id')
         return qs
 
     def get_context_data(self, **kwargs):
@@ -489,7 +489,7 @@ class UserTopics(PaginatorMixin, generic.ListView):
         qs = super(UserTopics, self).get_queryset()
         qs = qs.filter(user=self.user)
         qs = perms.filter_topics(self.user, qs)
-        qs = qs.order_by('-updated', '-created')
+        qs = qs.order_by('-updated', '-created', '-id')
         return qs
 
     def get_context_data(self, **kwargs):
@@ -644,7 +644,8 @@ class TopicPollVoteView(generic.UpdateView):
 
     def form_valid(self, form):
         # already voted
-        if not pybb_topic_poll_not_voted(self.object, self.request.user):
+        if not perms.may_vote_in_topic(self.request.user, self.object) or \
+           not pybb_topic_poll_not_voted(self.object, self.request.user):
             return HttpResponseForbidden()
 
         answers = form.cleaned_data['answers']
@@ -713,10 +714,23 @@ def block_user(request, username):
     if 'block_and_delete_messages' in request.POST:
         # individually delete each post and empty topic to fire method
         # with forum/topic counters recalculation
-        for p in Post.objects.filter(user=user):
-            p.delete()
-        for t in Topic.objects.annotate(cnt=Count('posts')).filter(cnt=0):
-            t.delete()
+        posts = Post.objects.filter(user=user)
+        topics = posts.values('topic_id').distinct()
+        forums = posts.values('topic__forum_id').distinct()
+        posts.delete()
+        Topic.objects.filter(user=user).delete()
+        for t in topics:
+            try:
+                Topic.objects.get(id=t['topic_id']).update_counters()
+            except Topic.DoesNotExist:
+                pass
+        for f in forums:
+            try:
+                Forum.objects.get(id=f['topic__forum_id']).update_counters()
+            except Forum.DoesNotExist:
+                pass
+
+
     msg = _('User successfuly blocked')
     messages.success(request, msg, fail_silently=True)
     return redirect('pybb:index')
